from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('gift_subtype')
class GiftSubtype(Entity):
    LIST_URL = '/gift/v1/giftsubtypes'



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('constituent_online_presence')
class ConstituentOnlinePresence(Entity):
    LIST_URL = '/constituent/v1/constituents/{id}/onlinepresences?include_inactive=false'



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('custom_field_category')
class CustomFieldCategory(Entity):
    LIST_URL = '/constituent/v1/constituents/customfields/categories/details/'



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('relationship')
class Relationship(Entity):
    LIST_URL = '/constituent/v1/relationships/'
    CREATE_URL = '/constituent/v1/relationships/'
    GET_URL = '/constituent/v1/constituents/relationships/{id}'
    UPDATE_URL = '/constituent/v1/relationships/{id}'
    DELETE_URL = '/constituent/v1/relationships/{id}'



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('suffix')
class Suffix(Entity):
    LIST_URL = '/constituent/v1/suffixes'



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('phone')
class Phone(Entity):
    LIST_URL = '/constituent/v1/phones/'
    CREATE_URL = '/constituent/v1/phones/'
    GET_URL = '/constituent/v1/constituents/phones/{id}'
    UPDATE_URL = '/constituent/v1/phones/{id}'
    DELETE_URL = '/constituent/v1/phones/{id}'
    TYPES_URL = 'constituent/v1/phonetypes'



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('custom_field')
class CustomField(Entity):
    LIST_URL = '/constituent/v1/constituents/customfields/'
    CREATE_URL = '/constituent/v1/constituents/customfields/'
    UPDATE_URL = '/constituent/v1/constituents/customfields/{id}/'
    DELETE_URL = '/constituent/v1/constituents/customfields/{id}/'



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('gift_batch')
class GiftBatch(Entity):
    LIST_URL = '/gift-batch/v1/giftbatches'
    CREATE_URL = '/gift-batch/v1/giftbatches'
    DELETE_URL = '/gift-batch/v1/giftbatches/{id}'



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('fundraising_campaign')
class FundraisingCampaign(Entity):
    LIST_URL = '/fundraising/v1/campaigns/'
    GET_URL = '/fundraising/v1/campaigns/{id}'



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('gift_batch_gift')
class GiftBatchGift(Entity):
    CREATE_URL = '/gift/v1/giftbatches/{id}/gifts'



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('event_fee')
class EventFee(Entity):
    LIST_URL = '/event/v1/events/{id}/eventfees'
    CREATE_URL = '/event/v1/events/{id}/eventfees'
    UPDATE_URL = '/event/v1/eventfees/{id}'
    DELETE_URL = '/event/v1/eventfees/{id}'



from typing import Dict
from sky_api_client.exceptions.exception import MethodNotDefined, InvalidArguments
from sky_api_client import client


class Entity(object):
    CREATE_URL = None
    UPDATE_URL = None
    GET_URL = None
    LIST_URL = None
    DELETE_URL = None
    SEARCH_URL = None
    TYPES_URL = None

    def __init__(self, api: client) -> None:
        self._api = api

    def list(self, id=None, parent_id=None, params=None):
        if self.LIST_URL:
            if 'id' in self.LIST_URL and id is None:
                raise InvalidArguments()
            elif 'parent_id' in self.LIST_URL and parent_id is None:
                raise InvalidArguments()
            else:
                return self._api.request(method='GET', path=self.LIST_URL.format(id=id), params=params)
        raise MethodNotDefined('List')

    def types(self):
        if self.TYPES_URL:
            return self._api.request(method='GET', path=self.TYPES_URL).get('value', [])
        raise MethodNotDefined('Types')

    def get(self, id: str, parent_id=''):
        if self.GET_URL:
            return self._api.request(method='GET', path=self.GET_URL.format(id=id, parent_id=parent_id))
        raise MethodNotDefined('Get')

    def create(self, data: Dict[str, str], id=''):
        if self.CREATE_URL:
            return self._api.request(method='POST', path=self.CREATE_URL.format(id=id), data=data)
        raise MethodNotDefined('Create')

    def update(self, id: str, data: Dict[str, str], parent_id=''):
        if self.UPDATE_URL:
            return self._api.request(method='PATCH', path=self.UPDATE_URL.format(id=id, parent_id=parent_id), data=data)
        raise MethodNotDefined('Update')

    def delete(self, id: str, parent_id=''):
        if self.DELETE_URL:
            return self._api.request(method='DELETE', path=self.DELETE_URL.format(id=id, parent_id=parent_id))
        raise MethodNotDefined('Delete')

    def search(self, search_text: str):
        if self.SEARCH_URL:
            return self._api.request(method='GET', path=self.SEARCH_URL.format(search_text=search_text))
        raise MethodNotDefined('Search')



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('webhook_subscription')
class WebhookSubscription(Entity):
    LIST_URL = '/webhook/v1/subscriptions/'
    CREATE_URL = '/webhook/v1/subscriptions/'
    GET_URL = '/webhook/v1/subscriptions/{id}/'
    DELETE_URL = '/webhook/v1/subscriptions/{id}/'



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('constituent_solicit_code')
class ConstituentSolicitCode(Entity):
    LIST_URL = '/commpref/v1/constituents/{id}/constituentsolicitcodes'



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('event')
class Event(Entity):
    LIST_URL = '/event/v1/eventlist'
    CREATE_URL = '/event/v1/events'
    GET_URL = '/event/v1/events/{id}'
    UPDATE_URL = '/event/v1/events/{id}'
    DELETE_URL = '/event/v1/events/{id}'



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('constituent_custom_field')
class ConstituentCustomField(Entity):
    LIST_URL = '/constituent/v1/constituents/{id}/customfields'



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('fundraising_package')
class FundraisingPackage(Entity):
    LIST_URL = '/fundraising/v1/packages/'
    GET_URL = '/fundraising/v1/packages/{id}'



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('table_entry')
class TableEntry(Entity):
    LIST_URL = '/nxt-data-integration/v1/re/codetables/{id}/tableentries/'
    CREATE_URL = '/nxt-data-integration/v1/re/codetables/{id}/tableentries/'
    GET_URL = '/nxt-data-integration/v1/re/codetables/{parent_id}/tableentries/{id}'
    UPDATE_URL = '/nxt-data-integration/v1/re/codetables/{parent_id}/tableentries/{id}/'
    DELETE_URL = '/nxt-data-integration/v1/re/codetables/{parent_id}/tableentries/{id}/'



import time
from typing import Dict, Any
import requests
from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry
from sky_api_client.exceptions.exception import GeneralError

BASE_URL = 'https://api.sky.blackbaud.com'
DEFAULT_TIMEOUT = 300


class SkyApiClient(object):
    def __init__(self, subscription_key: str, access_token: str) -> None:
        self.subscription_key = subscription_key
        self.access_token = access_token

    def request(self, method: str, path: str, data: Dict[str, str] = None, params: Dict[str, Any] = None):
        response = requests.request(
            method=method,
            url='{}/{}'.format(BASE_URL, path),
            timeout=DEFAULT_TIMEOUT,
            headers=self.get_headers(),
            json=data or {},
            params=params,
        )

        if response.status_code == 429:
            sleep_seconds = response.headers.get('Retry-After', 3)
            time.sleep(int(sleep_seconds))

            # retry the failed request. In the worst case, we'll exceed recursion limit and the job will fail
            return self.request(method, path, data=data)

        if response.ok:
            try:
                return response.json()
            except Exception:
                return 'Success'
        raise GeneralError(response.text)

    def get_headers(self):
        return {
            'Bb-Api-Subscription-Key': self.subscription_key,
            'Authorization': 'Bearer {access_token}'.format(access_token=self.access_token),
            'Content-Type': 'application/json',
        }


class SkyApi(object):
    def __init__(self, subscription_key: str, access_token: str) -> None:
        self._api = SkyApiClient(subscription_key, access_token)

    def __getattr__(self, attr: str) -> Entity:
        entity_class = EntityRegistry.get_class(attr)
        if entity_class:
            return entity_class(api=self._api)



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('event_participant_donation')
class EventParticipantFeePayment(Entity):
    LIST_URL = '/event/v1/participants/{id}/donations'
    CREATE_URL = '/event/v1/participants/{id}/donations'
    DELETE_URL = '/event/v1/participantdonations/{id}'



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry
from sky_api_client.exceptions.exception import MethodNotDefined


@EntityRegistry.register('solicit_code')
class SolicitCode(Entity):
    CREATE_URL = '/commpref/v1/constituentsolicitcodes'
    GET_URL = '/commpref/v1/constituentsolicitcodes/{id}'
    UPDATE_URL = '/commpref/v1/constituentsolicitcodes/{id}'
    DELETE_URL = '/commpref/v1/constituentsolicitcodes/{id}'
    TYPES_URL = '/commpref/v1/solicitcodes'

    def types(self):
        if self.TYPES_URL:
            results = self._api.request(method='GET', path=self.TYPES_URL).get('value', [])
            return [result['description'] for result in results]
        raise MethodNotDefined('Types')



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('constituent_code')
class ConstituentCode(Entity):
    LIST_URL = '/constituent/v1/constituents/{id}/constituentcodes'
    GET_URL = '/constituent/v1/constituents/constituentcodes/{id}'
    CREATE_URL = '/constituent/v1/constituentcodes/'
    UPDATE_URL = '/constituent/v1/constituentcodes/{id}'
    DELETE_URL = '/constituent/v1/constituentcodes/{id}'
    TYPES_URL = '/constituent/v1/constituentcodetypes'



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('event_participant_levels')
class EventParticipantLevels(Entity):
    LIST_URL = '/event/v1/participationlevels'
    CREATE_URL = '/event/v1/participationlevels'
    UPDATE_URL = '/event/v1/participationlevels/{id}'
    DELETE_URL = '/event/v1/participationlevels/{id}'



from typing import Callable


class EntityRegistry(object):
    registry = {}

    @classmethod
    def register(cls, entity: str) -> Callable:
        def inner_wrapper(wrapped_class):
            cls.registry[entity] = wrapped_class
            return wrapped_class

        return inner_wrapper

    @classmethod
    def get_class(cls, entity: str, **kwargs):
        if entity not in cls.registry:
            return None
        exec_class = cls.registry[entity]
        return exec_class



from sky_api_client.client import SkyApi  # noqa: F401
from sky_api_client.entity import address  # noqa: F401
from sky_api_client.entity import code_table  # noqa: F401
from sky_api_client.entity import constituent  # noqa: F401
from sky_api_client.entity import custom_field  # noqa: F401
from sky_api_client.entity import custom_field_categories  # noqa: F401
from sky_api_client.entity import education  # noqa: F401
from sky_api_client.entity import email_addresses  # noqa: F401
from sky_api_client.entity import event  # noqa: F401
from sky_api_client.entity import event_fee  # noqa: F401
from sky_api_client.entity import event_participant  # noqa: F401
from sky_api_client.entity import event_participant_fee  # noqa: F401
from sky_api_client.entity import event_participant_fee_payment  # noqa: F401
from sky_api_client.entity import event_participant_donation  # noqa: F401
from sky_api_client.entity import event_participation_levels  # noqa: F401
from sky_api_client.entity import gender  # noqa: F401
from sky_api_client.entity import gift  # noqa: F401
from sky_api_client.entity import gift_batch  # noqa: F401
from sky_api_client.entity import gift_batch_gift  # noqa: F401
from sky_api_client.entity import online_presence  # noqa: F401
from sky_api_client.entity import phone  # noqa: F401
from sky_api_client.entity import suffix  # noqa: F401
from sky_api_client.entity import title  # noqa: F401
from sky_api_client.entity import relationship  # noqa: F401
from sky_api_client.entity import table_entry  # noqa: F401
from sky_api_client.entity import webhook_subscription  # noqa: F401
from sky_api_client.entity import constituent_address  # noqa: F401
from sky_api_client.entity import constituent_custom_field  # noqa: F401
from sky_api_client.entity import constituent_education  # noqa: F401
from sky_api_client.entity import constituent_email_address  # noqa: F401
from sky_api_client.entity import constituent_online_presence  # noqa: F401
from sky_api_client.entity import constituent_phone  # noqa: F401
from sky_api_client.entity import constituent_code  # noqa: F401
from sky_api_client.entity import constituent_relationship  # noqa: F401
from sky_api_client.entity import gift_subtype  # noqa: F401
from sky_api_client.entity import fundraising_appeal  # noqa: F401
from sky_api_client.entity import fundraising_fund  # noqa: F401
from sky_api_client.entity import fundraising_campaign  # noqa: F401
from sky_api_client.entity import fundraising_package  # noqa: F401
from sky_api_client.entity import constituent_solicit_code  # noqa: F401
from sky_api_client.entity import solicit_code  # noqa: F401



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry
from sky_api_client.exceptions.exception import MethodNotDefined


@EntityRegistry.register('education')
class Education(Entity):
    LIST_URL = '/constituent/v1/educations/'
    CREATE_URL = '/constituent/v1/educations/'
    GET_URL = '/constituent/v1/constituents/educations/{id}'
    UPDATE_URL = '/constituent/v1/educations/{id}'
    DELETE_URL = '/constituent/v1/educations/{id}'
    SUBJECTS_URL = '/constituent/v1/educations/subjects'
    DEGREES_URL = '/constituent/v1/educations/degrees'

    def subjects(self):
        if self.SUBJECTS_URL:
            return self._api.request(method='GET', path=self.SUBJECTS_URL).get('value', [])
        raise MethodNotDefined('subjects')

    def degrees(self):
        if self.DEGREES_URL:
            return self._api.request(method='GET', path=self.DEGREES_URL).get('value', [])
        raise MethodNotDefined('degrees')



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('gift')
class Gift(Entity):
    LIST_URL = '/gift/v1/gifts'
    CREATE_URL = '/gift/v1/gifts'
    GET_URL = '/gift/v1/gifts/{id}'
    UPDATE_URL = '/gift/v1/gifts/{id}/'
    DELETE_URL = '/gift/v1/gifts/{id}/'



import setuptools

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name='sky_api_client',
    version='3.2.5',
    author='Uddesh Jain',
    author_email='uddesh@almabase.com',
    description='Python client for RENXT APIs',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://bitbucket.org/kalyanvarma/skyapi',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    packages=setuptools.find_packages(),
    python_requires='>=3.5',
)



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('fundraising_fund')
class FundraisingFund(Entity):
    LIST_URL = '/fundraising/v1/funds/'
    GET_URL = '/fundraising/v1/funds/{id}'



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('email_addresses')
class EmailAddress(Entity):
    LIST_URL = '/constituent/v1/emailaddresses/'
    CREATE_URL = '/constituent/v1/emailaddresses/'
    UPDATE_URL = '/constituent/v1/emailaddresses/{id}'
    DELETE_URL = '/constituent/v1/emailaddresses/{id}'
    TYPES_URL = 'constituent/v1/emailaddresstypes'



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('online_presence')
class OnlinePresence(Entity):
    LIST_URL = '/constituent/v1/onlinepresences?include_inactive=false'
    CREATE_URL = '/constituent/v1/onlinepresences/'
    GET_URL = '/constituent/v1/constituents/onlinepresences/{id}'
    UPDATE_URL = '/constituent/v1/onlinepresences/{id}'
    DELETE_URL = '/constituent/v1/onlinepresences/{id}'
    TYPES_URL = 'constituent/v1/onlinepresencetypes'



class GeneralError(Exception):
    pass


class MethodNotDefined(Exception):
    def __init__(self, method: str) -> None:
        self.method = method
        super().__init__('{} method is not available for this entity'.format(self.method))


class InvalidArguments(Exception):
    def __init__(self) -> None:
        super().__init__('Invalid Arguments')



from sky_api_client.entity.base import Entity
from sky_api_client.entity.registry import EntityRegistry


@EntityRegistry.register('constituent')
class Constituent(Entity):
    LIST_URL = '/constituent/v1/constituents/'
    CREATE_URL = '/constituent/v1/constituents/'
    GET_URL = '/constituent/v1/constituents/{id}'
    UPDATE_URL = '/constituent/v1/constituents/{id}'
    DELETE_URL = '/constituent/v1/constituents/{id}'
    SEARCH_URL = '/constituent/v1/constituents/search?search_text={search_text}'
